//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	avlmap/map
// homepage	http://avlmap.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	avlmap@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		map.h
//
// purpose	Define resources for the Data Mapping library package.
//-----------------------------------------------------------------------------
#ifndef __MAP_H__
#define __MAP_H__

#include <ctype.h>

#include <avlmap/avl.h>

//-----------------------------------------------------------------------------
// struct	map_root
// type		MAP
//
// purpose	This structure is the anchor point for a data mapping.
//		The pointer to it is the mapping handle.
//-----------------------------------------------------------------------------
struct map_root {
    int ( *	cmp_func	)()	;
    int		options		;
    int		key_type	;
    avl_root	avl_tree	;
};
typedef struct map_root		map_t	;
typedef struct map_root		MAP_T	;
typedef struct map_root *	MAP	;
typedef struct map_root *	map_p	;
typedef struct map_root *	MAP_P	;

#define MAP_OPT_LOWER	1
#define MAP_OPT_UPPER	2
#define MAP_OPT_DIGITS	4
#define MAP_OPT_SPACES	8
#define MAP_OPT_PATHS	16

//-----------------------------------------------------------------------------
// struct	map_mixed
//
// purpose	This structure holds data for each node.
//-----------------------------------------------------------------------------
struct map_mixed {
    union map_data {
	union map_scalar {
	    signed char		sc	;
	    unsigned char	uc	;
	    signed short	ss	;
	    unsigned short	us	;
	    signed int		si	;
	    unsigned int	ui	;
	    signed long		sl	;
	    unsigned long	ul	;
	    signed long long	sll	;
	    unsigned long long	ull	;
	    double		d	;
	    long double		ld	;
	    void *		ptr	;
	    MAP			map	;
	}		scalar	;
	struct map_array {
	    void *		ptr	;
	    size_t		len	;
	    char		dup	;
	}		array	;
    }		mix	;
    int		type	;
};
typedef struct map_mixed	map_mixed	;

//-----------------------------------------------------------------------------
// Define various macros to describe generic type properties.
//-----------------------------------------------------------------------------
#define MAP_DATA_SIGNED		0x00000100
#define MAP_DATA_UNSIGNED	0x00000200

#define MAP_DATA_INTEGER	0x00000400
#define MAP_DATA_FLOAT		0x00000800
#define MAP_DATA_OTHER		0x00001000

#define MAP_DATA_SCALAR		0x00002000
#define MAP_DATA_ARRAY		0x00004000
#define MAP_DATA_STRING		MAP_DATA_ARRAY

//-----------------------------------------------------------------------------
// Specific data type codes are defined as their size values.  This results in
// equivalent types being assigned the same type code on any given platform.
//-----------------------------------------------------------------------------
#define MAP_DATA_DOUBLE		(sizeof(double))
#define MAP_DATA_LONG_DOUBLE	(sizeof(long double))

#define MAP_DATA_CHAR		(sizeof(char))
#define MAP_DATA_SHORT		(sizeof(short))
#define MAP_DATA_INT		(sizeof(int))
#define MAP_DATA_LONG		(sizeof(long))
#define MAP_DATA_LONG_LONG	(sizeof(long long))

#define MAP_DATA_POINTER	(sizeof(void *))
#define MAP_DATA_MAPPTR		(sizeof(map_p))

//-----------------------------------------------------------------------------
// Define macros that can be used to isolate generic properties of a type.
//-----------------------------------------------------------------------------
#define map_type_is_signed(t)		\
	(((t)&((MAP_DATA_SIGNED)|(MAP_DATA_UNSIGNED)))==(MAP_DATA_SIGNED))
#define map_type_is_unsigned(t)		\
	(((t)&((MAP_DATA_SIGNED)|(MAP_DATA_UNSIGNED)))==(MAP_DATA_UNSIGNED))

#define map_type_is_integer(t)		\
	(((t)&((MAP_DATA_INTEGER)|(MAP_DATA_FLOAT)|(MAP_DATA_OTHER)))==(MAP_DATA_INTEGER))
#define map_type_is_float(t)		\
	(((t)&((MAP_DATA_INTEGER)|(MAP_DATA_FLOAT)|(MAP_DATA_OTHER)))==(MAP_DATA_FLOAT))

#define map_type_is_scalar(t)		\
	(((t)&((MAP_DATA_SCALAR)|(MAP_DATA_ARRAY)))==(MAP_DATA_SCALAR))
#define map_type_is_array(t)		\
	(((t)&((MAP_DATA_SCALAR)|(MAP_DATA_ARRAY)))==(MAP_DATA_ARRAY))
#define map_type_is_string(t)		\
	(((t)&((MAP_DATA_SCALAR)|(MAP_DATA_ARRAY)))==(MAP_DATA_ARRAY))

#define map_type_sizeof(t)	((t)&(0x000000ff))

//-----------------------------------------------------------------------------
// Define macros for each individual type constructed from its properties.
//-----------------------------------------------------------------------------
#define MAP_DATA_SC	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_CHAR
#define MAP_DATA_UC	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_CHAR
#define MAP_DATA_SS	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_SHORT
#define MAP_DATA_US	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_SHORT
#define MAP_DATA_SI	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_INT
#define MAP_DATA_UI	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_INT
#define MAP_DATA_SL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG
#define MAP_DATA_UL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG
#define MAP_DATA_SLL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_ULL	MAP_DATA_SCALAR+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_C	MAP_DATA_UC

#define MAP_DATA_D	MAP_DATA_SCALAR+MAP_DATA_FLOAT+MAP_DATA_DOUBLE
#define MAP_DATA_LD	MAP_DATA_SCALAR+MAP_DATA_FLOAT+MAP_DATA_LONG_DOUBLE

#define MAP_DATA_PTR	MAP_DATA_SCALAR+MAP_DATA_OTHER+MAP_DATA_POINTER
#define MAP_DATA_MAP	MAP_DATA_SCALAR+MAP_DATA_OTHER+MAP_DATA_MAPPTR

#define MAP_DATA_SCA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_CHAR
#define MAP_DATA_UCA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_CHAR
#define MAP_DATA_SSA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_SHORT
#define MAP_DATA_USA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_SHORT
#define MAP_DATA_SIA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_INT
#define MAP_DATA_UIA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_INT
#define MAP_DATA_SLA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG
#define MAP_DATA_ULA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG
#define MAP_DATA_SLLA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_ULLA	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_CA	MAP_DATA_UCA

#define MAP_DATA_DA	MAP_DATA_ARRAY+MAP_DATA_FLOAT+MAP_DATA_DOUBLE
#define MAP_DATA_LDA	MAP_DATA_ARRAY+MAP_DATA_FLOAT+MAP_DATA_LONG_DOUBLE

#define MAP_DATA_SCS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_CHAR
#define MAP_DATA_UCS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_CHAR
#define MAP_DATA_SSS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_SHORT
#define MAP_DATA_USS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_SHORT
#define MAP_DATA_SIS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_INT
#define MAP_DATA_UIS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_INT
#define MAP_DATA_SLS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG
#define MAP_DATA_ULS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG
#define MAP_DATA_SLLS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_SIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_ULLS	MAP_DATA_ARRAY+MAP_DATA_INTEGER+MAP_DATA_UNSIGNED+MAP_DATA_LONG_LONG
#define MAP_DATA_CS	MAP_DATA_UCS
#define MAP_DATA_STR	MAP_DATA_UCS

#define MAP_DATA_DS	MAP_DATA_ARRAY+MAP_DATA_FLOAT+MAP_DATA_DOUBLE
#define MAP_DATA_LDS	MAP_DATA_ARRAY+MAP_DATA_FLOAT+MAP_DATA_LONG_DOUBLE

//-----------------------------------------------------------------------------
// macro	map_loop_forward
//
// purpose	Start a program loop which steps forward over all the
//		elements in a mapping, starting at the beginning.
//
// arguments	1 (MAP) which mapping
//
// usage	map_loop_forward( map ) { /* body of loop */ }
//
//		In the body of the loop, place the appropriate call to get
//		the data type and data that for that loop iteratiin.
//-----------------------------------------------------------------------------
#define map_loop_forward(m) for(map_unselect((m));map_forward((m))>=0;)

//-----------------------------------------------------------------------------
// macro	map_loop_reverse
//
// purpose	Start a program loop which steps in reverse over all the
//		elements in a mapping, starting at the end.
//
// arguments	1 (MAP) which mapping
//
// usage	map_loop_reverse( map ) { /* body of loop */ }
//
//		In the body of the loop, place the appropriate call to get
//		the data type and data that for that loop iteratiin.
//-----------------------------------------------------------------------------
#define map_loop_reverse(m) for(map_unselect((m));map_reverse((m))>=0;)

//-----------------------------------------------------------------------------
// macro	map_str_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_digits map_ca_cmp_digits
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_digits map_ca_cmp_digits
//-----------------------------------------------------------------------------
// macro	map_str_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_feature map_ca_cmp_feature
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_feature map_ca_cmp_feature
//-----------------------------------------------------------------------------
// macro	map_str_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_func map_ca_cmp_func
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_func map_ca_cmp_func
//-----------------------------------------------------------------------------
// macro	map_str_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_func_opt map_ca_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_func_opt map_ca_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_str_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_lower map_ca_cmp_lower
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_lower map_ca_cmp_lower
//-----------------------------------------------------------------------------
// macro	map_str_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_paths map_ca_cmp_paths
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_paths map_ca_cmp_paths
//-----------------------------------------------------------------------------
// macro	map_str_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_spaces map_ca_cmp_spaces
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_spaces map_ca_cmp_spaces
//-----------------------------------------------------------------------------
// macro	map_str_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_upper map_ca_cmp_upper
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_upper map_ca_cmp_upper
//-----------------------------------------------------------------------------
// macro	map_str_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_str_fetch_copy map_cs_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_cs_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_cs_fetch_copy map_ca_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_str_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(char *) pointer to duplicate data string array
//		(char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_str_fetch_dup map_cs_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_cs_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(char *) pointer to duplicate data string array
//		(char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_cs_fetch_dup map_ca_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_str_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_str_fetch_len map_cs_fetch_len
//-----------------------------------------------------------------------------
// macro	map_cs_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_cs_fetch_len map_ca_fetch_len
//-----------------------------------------------------------------------------
// macro	map_str_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const char *) pointer to data string array
//		(const char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_str_fetch_ptr map_cs_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_cs_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const char *) pointer to data string array
//		(const char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_cs_fetch_ptr map_ca_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_str_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_exact(m,k) map_ca_find((m),(k),~(size_t)0,0)
//-----------------------------------------------------------------------------
// macro	map_cs_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_cs_find_exact(m,k) map_ca_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_ca_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ca_find_exact(m,k,l) map_ca_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_str_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_exact_lo(m,k) map_ca_find((m),(k),~(size_t)0,-1)
//-----------------------------------------------------------------------------
// macro	map_cs_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_cs_find_exact_lo(m,k) map_ca_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_ca_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ca_find_exact_lo(m,k,l) map_ca_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_str_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_lo_or_near(m,k) map_ca_find((m),(k),~(size_t)0,-2)
//-----------------------------------------------------------------------------
// macro	map_cs_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_cs_find_lo_or_near(m,k) map_ca_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_ca_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ca_find_lo_or_near(m,k,l) map_ca_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_str_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_exact_hi(m,k) map_ca_find((m),(k),~(size_t)0,1)
//-----------------------------------------------------------------------------
// macro	map_cs_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_cs_find_exact_hi(m,k) map_ca_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_ca_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ca_find_exact_hi(m,k,l) map_ca_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_str_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_hi_or_near(m,k) map_ca_find((m),(k),~(size_t)0,2)
//-----------------------------------------------------------------------------
// macro	map_cs_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_cs_find_hi_or_near(m,k) map_ca_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_ca_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ca_find_hi_or_near(m,k,l) map_ca_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_str_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find(m,k,d) map_ca_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_cs_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_cs_find(m,k,d) map_ca_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_str_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_str_insert(m,k) map_ca_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_cs_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_cs_insert(m,k) map_ca_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_str_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_str_insert_dup(m,k) map_ca_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_cs_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_cs_insert_dup(m,k) map_ca_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_str_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_copy map_ca_key_copy
//-----------------------------------------------------------------------------
// macro	map_cs_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_cs_key_copy map_ca_key_copy
//-----------------------------------------------------------------------------
// macro	map_str_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(char *) pointer to key duplicate copy
//		(char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_dup map_ca_key_dup
//-----------------------------------------------------------------------------
// macro	map_cs_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(char *) pointer to key duplicate copy
//		(char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_cs_key_dup map_ca_key_dup
//-----------------------------------------------------------------------------
// macro	map_str_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_len map_ca_key_len
//-----------------------------------------------------------------------------
// macro	map_cs_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_cs_key_len map_ca_key_len
//-----------------------------------------------------------------------------
// macro	map_str_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const char *) pointer to shared key
//		(const char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_ptr map_ca_key_ptr
//-----------------------------------------------------------------------------
// macro	map_cs_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const char *) pointer to shared key
//		(const char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_cs_key_ptr map_ca_key_ptr
//-----------------------------------------------------------------------------
// macro	map_str_new
//
// purpose	Create a new mapping with a string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_str_new map_ca_new
//-----------------------------------------------------------------------------
// macro	map_cs_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_cs_new map_ca_new
//-----------------------------------------------------------------------------
// macro	map_str_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_str_store(m,d) map_ca_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_cs_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_cs_store(m,d) map_ca_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_str_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_str_store_share(m,d) map_ca_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_cs_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_cs_store_share(m,d) map_ca_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_d_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_d_find_exact(m,k) map_d_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_d_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_d_find_exact_lo(m,k) map_d_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_d_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_d_find_lo_or_near(m,k) map_d_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_d_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_d_find_exact_hi(m,k) map_d_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_d_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_d_find_hi_or_near(m,k) map_d_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_ld_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ld_find_exact(m,k) map_ld_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_ld_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ld_find_exact_lo(m,k) map_ld_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_ld_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ld_find_lo_or_near(m,k) map_ld_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_ld_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ld_find_exact_hi(m,k) map_ld_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_ld_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ld_find_hi_or_near(m,k) map_ld_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_ptr_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ptr_find_exact(m,k) map_ptr_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_ptr_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ptr_find_exact_lo(m,k) map_ptr_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_ptr_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ptr_find_lo_or_near(m,k) map_ptr_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_ptr_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ptr_find_exact_hi(m,k) map_ptr_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_ptr_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ptr_find_hi_or_near(m,k) map_ptr_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_digits map_sca_cmp_digits
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_feature map_sca_cmp_feature
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_func map_sca_cmp_func
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_func_opt map_sca_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_lower map_sca_cmp_lower
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_paths map_sca_cmp_paths
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_spaces map_sca_cmp_spaces
//-----------------------------------------------------------------------------
// macro	map_scs_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_scs_cmp_upper map_sca_cmp_upper
//-----------------------------------------------------------------------------
// macro	map_scs_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed char *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_scs_fetch_copy map_sca_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_scs_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed char *) pointer to duplicate data string array
//		(signed char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_scs_fetch_dup map_sca_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_scs_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_scs_fetch_len map_sca_fetch_len
//-----------------------------------------------------------------------------
// macro	map_scs_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed char *) pointer to data string array
//		(const signed char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_scs_fetch_ptr map_sca_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_scs_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_scs_find_exact(m,k) map_sca_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_sca_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sca_find_exact(m,k,l) map_sca_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_scs_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_scs_find_exact_lo(m,k) map_sca_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_sca_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sca_find_exact_lo(m,k,l) map_sca_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_scs_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_scs_find_lo_or_near(m,k) map_sca_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_sca_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sca_find_lo_or_near(m,k,l) map_sca_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_scs_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_scs_find_exact_hi(m,k) map_sca_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_sca_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sca_find_exact_hi(m,k,l) map_sca_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_scs_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_scs_find_hi_or_near(m,k) map_sca_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_sca_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sca_find_hi_or_near(m,k,l) map_sca_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_scs_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_scs_find(m,k,d) map_sca_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_scs_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_scs_insert(m,k) map_sca_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_scs_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_scs_insert_dup(m,k) map_sca_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_scs_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed char *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_scs_key_copy map_sca_key_copy
//-----------------------------------------------------------------------------
// macro	map_scs_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed char *) pointer to key duplicate copy
//		(signed char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_scs_key_dup map_sca_key_dup
//-----------------------------------------------------------------------------
// macro	map_scs_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_scs_key_len map_sca_key_len
//-----------------------------------------------------------------------------
// macro	map_scs_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed char *) pointer to shared key
//		(const signed char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_scs_key_ptr map_sca_key_ptr
//-----------------------------------------------------------------------------
// macro	map_scs_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_scs_new map_sca_new
//-----------------------------------------------------------------------------
// macro	map_scs_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_scs_store(m,d) map_sca_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_scs_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed char *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_scs_store_share(m,d) map_sca_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_si_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_si_find_exact(m,k) map_si_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_si_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_si_find_exact_lo(m,k) map_si_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_si_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_si_find_lo_or_near(m,k) map_si_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_si_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_si_find_exact_hi(m,k) map_si_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_si_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_si_find_hi_or_near(m,k) map_si_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_sis_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_sis_cmp_func map_sia_cmp_func
//-----------------------------------------------------------------------------
// macro	map_sis_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_sis_cmp_func_opt map_sia_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_sis_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_sis_fetch_copy map_sia_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_sis_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed int *) pointer to duplicate data string array
//		(signed int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_sis_fetch_dup map_sia_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_sis_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_sis_fetch_len map_sia_fetch_len
//-----------------------------------------------------------------------------
// macro	map_sis_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed int *) pointer to data string array
//		(const signed int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_sis_fetch_ptr map_sia_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_sis_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sis_find_exact(m,k) map_sia_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_sia_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sia_find_exact(m,k,l) map_sia_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_sis_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sis_find_exact_lo(m,k) map_sia_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_sia_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sia_find_exact_lo(m,k,l) map_sia_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_sis_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sis_find_lo_or_near(m,k) map_sia_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_sia_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sia_find_lo_or_near(m,k,l) map_sia_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_sis_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sis_find_exact_hi(m,k) map_sia_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_sia_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sia_find_exact_hi(m,k,l) map_sia_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_sis_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sis_find_hi_or_near(m,k) map_sia_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_sia_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sia_find_hi_or_near(m,k,l) map_sia_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_sis_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sis_find(m,k,d) map_sia_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_sis_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_sis_insert(m,k) map_sia_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sis_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_sis_insert_dup(m,k) map_sia_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sis_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sis_key_copy map_sia_key_copy
//-----------------------------------------------------------------------------
// macro	map_sis_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed int *) pointer to key duplicate copy
//		(signed int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sis_key_dup map_sia_key_dup
//-----------------------------------------------------------------------------
// macro	map_sis_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sis_key_len map_sia_key_len
//-----------------------------------------------------------------------------
// macro	map_sis_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed int *) pointer to shared key
//		(const signed int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sis_key_ptr map_sia_key_ptr
//-----------------------------------------------------------------------------
// macro	map_sis_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_sis_new map_sia_new
//-----------------------------------------------------------------------------
// macro	map_sis_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_sis_store(m,d) map_sia_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sis_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_sis_store_share(m,d) map_sia_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sl_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sl_find_exact(m,k) map_sl_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_sl_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sl_find_exact_lo(m,k) map_sl_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_sl_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sl_find_lo_or_near(m,k) map_sl_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_sl_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sl_find_exact_hi(m,k) map_sl_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_sl_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sl_find_hi_or_near(m,k) map_sl_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_sls_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_sls_cmp_func map_sla_cmp_func
//-----------------------------------------------------------------------------
// macro	map_sls_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_sls_cmp_func_opt map_sla_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_sls_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_sls_fetch_copy map_sla_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_sls_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long *) pointer to duplicate data string array
//		(signed long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_sls_fetch_dup map_sla_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_sls_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_sls_fetch_len map_sla_fetch_len
//-----------------------------------------------------------------------------
// macro	map_sls_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long *) pointer to data string array
//		(const signed long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_sls_fetch_ptr map_sla_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_sls_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sls_find_exact(m,k) map_sla_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_sla_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sla_find_exact(m,k,l) map_sla_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_sls_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sls_find_exact_lo(m,k) map_sla_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_sla_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sla_find_exact_lo(m,k,l) map_sla_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_sls_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sls_find_lo_or_near(m,k) map_sla_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_sla_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sla_find_lo_or_near(m,k,l) map_sla_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_sls_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sls_find_exact_hi(m,k) map_sla_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_sla_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sla_find_exact_hi(m,k,l) map_sla_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_sls_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sls_find_hi_or_near(m,k) map_sla_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_sla_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sla_find_hi_or_near(m,k,l) map_sla_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_sls_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sls_find(m,k,d) map_sla_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_sls_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_sls_insert(m,k) map_sla_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sls_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_sls_insert_dup(m,k) map_sla_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sls_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sls_key_copy map_sla_key_copy
//-----------------------------------------------------------------------------
// macro	map_sls_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long *) pointer to key duplicate copy
//		(signed long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sls_key_dup map_sla_key_dup
//-----------------------------------------------------------------------------
// macro	map_sls_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sls_key_len map_sla_key_len
//-----------------------------------------------------------------------------
// macro	map_sls_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long *) pointer to shared key
//		(const signed long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sls_key_ptr map_sla_key_ptr
//-----------------------------------------------------------------------------
// macro	map_sls_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_sls_new map_sla_new
//-----------------------------------------------------------------------------
// macro	map_sls_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_sls_store(m,d) map_sla_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sls_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_sls_store_share(m,d) map_sla_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sll_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sll_find_exact(m,k) map_sll_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_sll_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sll_find_exact_lo(m,k) map_sll_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_sll_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sll_find_lo_or_near(m,k) map_sll_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_sll_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sll_find_exact_hi(m,k) map_sll_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_sll_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_sll_find_hi_or_near(m,k) map_sll_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_slls_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_slls_cmp_func map_slla_cmp_func
//-----------------------------------------------------------------------------
// macro	map_slls_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_slls_cmp_func_opt map_slla_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_slls_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_slls_fetch_copy map_slla_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_slls_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long long *) pointer to duplicate data string array
//		(signed long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_slls_fetch_dup map_slla_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_slls_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_slls_fetch_len map_slla_fetch_len
//-----------------------------------------------------------------------------
// macro	map_slls_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long long *) pointer to data string array
//		(const signed long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_slls_fetch_ptr map_slla_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_slls_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slls_find_exact(m,k) map_slla_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_slla_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slla_find_exact(m,k,l) map_slla_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_slls_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slls_find_exact_lo(m,k) map_slla_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_slla_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slla_find_exact_lo(m,k,l) map_slla_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_slls_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slls_find_lo_or_near(m,k) map_slla_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_slla_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slla_find_lo_or_near(m,k,l) map_slla_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_slls_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slls_find_exact_hi(m,k) map_slla_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_slla_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slla_find_exact_hi(m,k,l) map_slla_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_slls_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slls_find_hi_or_near(m,k) map_slla_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_slla_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slla_find_hi_or_near(m,k,l) map_slla_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_slls_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_slls_find(m,k,d) map_slla_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_slls_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_slls_insert(m,k) map_slla_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_slls_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_slls_insert_dup(m,k) map_slla_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_slls_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_slls_key_copy map_slla_key_copy
//-----------------------------------------------------------------------------
// macro	map_slls_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long long *) pointer to key duplicate copy
//		(signed long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_slls_key_dup map_slla_key_dup
//-----------------------------------------------------------------------------
// macro	map_slls_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_slls_key_len map_slla_key_len
//-----------------------------------------------------------------------------
// macro	map_slls_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long long *) pointer to shared key
//		(const signed long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_slls_key_ptr map_slla_key_ptr
//-----------------------------------------------------------------------------
// macro	map_slls_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_slls_new map_slla_new
//-----------------------------------------------------------------------------
// macro	map_slls_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_slls_store(m,d) map_slla_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_slls_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_slls_store_share(m,d) map_slla_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sss_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_sss_cmp_func map_ssa_cmp_func
//-----------------------------------------------------------------------------
// macro	map_sss_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_sss_cmp_func_opt map_ssa_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_sss_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed short *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_sss_fetch_copy map_ssa_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_sss_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed short *) pointer to duplicate data string array
//		(signed short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_sss_fetch_dup map_ssa_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_sss_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_sss_fetch_len map_ssa_fetch_len
//-----------------------------------------------------------------------------
// macro	map_sss_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed short *) pointer to data string array
//		(const signed short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_sss_fetch_ptr map_ssa_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_sss_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sss_find_exact(m,k) map_ssa_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_ssa_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ssa_find_exact(m,k,l) map_ssa_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_sss_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sss_find_exact_lo(m,k) map_ssa_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_ssa_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ssa_find_exact_lo(m,k,l) map_ssa_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_sss_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sss_find_lo_or_near(m,k) map_ssa_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_ssa_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ssa_find_lo_or_near(m,k,l) map_ssa_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_sss_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sss_find_exact_hi(m,k) map_ssa_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_ssa_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ssa_find_exact_hi(m,k,l) map_ssa_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_sss_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sss_find_hi_or_near(m,k) map_ssa_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_ssa_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ssa_find_hi_or_near(m,k,l) map_ssa_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_sss_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_sss_find(m,k,d) map_ssa_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_sss_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_sss_insert(m,k) map_ssa_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sss_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_sss_insert_dup(m,k) map_ssa_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sss_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed short *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sss_key_copy map_ssa_key_copy
//-----------------------------------------------------------------------------
// macro	map_sss_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed short *) pointer to key duplicate copy
//		(signed short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sss_key_dup map_ssa_key_dup
//-----------------------------------------------------------------------------
// macro	map_sss_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sss_key_len map_ssa_key_len
//-----------------------------------------------------------------------------
// macro	map_sss_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed short *) pointer to shared key
//		(const signed short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sss_key_ptr map_ssa_key_ptr
//-----------------------------------------------------------------------------
// macro	map_sss_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_sss_new map_ssa_new
//-----------------------------------------------------------------------------
// macro	map_sss_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_sss_store(m,d) map_ssa_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_sss_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed short *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_sss_store_share(m,d) map_ssa_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_digits map_uca_cmp_digits
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_feature map_uca_cmp_feature
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_func map_uca_cmp_func
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_func_opt map_uca_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_lower map_uca_cmp_lower
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_paths map_uca_cmp_paths
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_spaces map_uca_cmp_spaces
//-----------------------------------------------------------------------------
// macro	map_ucs_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ucs_cmp_upper map_uca_cmp_upper
//-----------------------------------------------------------------------------
// macro	map_ucs_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned char *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_ucs_fetch_copy map_uca_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_ucs_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned char *) pointer to duplicate data string array
//		(unsigned char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_ucs_fetch_dup map_uca_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_ucs_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_ucs_fetch_len map_uca_fetch_len
//-----------------------------------------------------------------------------
// macro	map_ucs_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned char *) pointer to data string array
//		(const unsigned char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_ucs_fetch_ptr map_uca_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_ucs_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ucs_find_exact(m,k) map_uca_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_uca_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uca_find_exact(m,k,l) map_uca_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_ucs_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ucs_find_exact_lo(m,k) map_uca_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_uca_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uca_find_exact_lo(m,k,l) map_uca_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_ucs_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ucs_find_lo_or_near(m,k) map_uca_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_uca_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uca_find_lo_or_near(m,k,l) map_uca_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_ucs_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ucs_find_exact_hi(m,k) map_uca_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_uca_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uca_find_exact_hi(m,k,l) map_uca_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_ucs_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ucs_find_hi_or_near(m,k) map_uca_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_uca_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uca_find_hi_or_near(m,k,l) map_uca_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_ucs_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ucs_find(m,k,d) map_uca_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_ucs_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_ucs_insert(m,k) map_uca_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ucs_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_ucs_insert_dup(m,k) map_uca_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ucs_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned char *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ucs_key_copy map_uca_key_copy
//-----------------------------------------------------------------------------
// macro	map_ucs_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned char *) pointer to key duplicate copy
//		(unsigned char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ucs_key_dup map_uca_key_dup
//-----------------------------------------------------------------------------
// macro	map_ucs_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ucs_key_len map_uca_key_len
//-----------------------------------------------------------------------------
// macro	map_ucs_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned char *) pointer to shared key
//		(const unsigned char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ucs_key_ptr map_uca_key_ptr
//-----------------------------------------------------------------------------
// macro	map_ucs_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_ucs_new map_uca_new
//-----------------------------------------------------------------------------
// macro	map_ucs_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_ucs_store(m,d) map_uca_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ucs_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned char *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_ucs_store_share(m,d) map_uca_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ui_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ui_find_exact(m,k) map_ui_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_ui_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ui_find_exact_lo(m,k) map_ui_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_ui_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ui_find_lo_or_near(m,k) map_ui_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_ui_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ui_find_exact_hi(m,k) map_ui_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_ui_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ui_find_hi_or_near(m,k) map_ui_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_uis_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_uis_cmp_func map_uia_cmp_func
//-----------------------------------------------------------------------------
// macro	map_uis_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_uis_cmp_func_opt map_uia_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_uis_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_uis_fetch_copy map_uia_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_uis_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned int *) pointer to duplicate data string array
//		(unsigned int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_uis_fetch_dup map_uia_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_uis_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_uis_fetch_len map_uia_fetch_len
//-----------------------------------------------------------------------------
// macro	map_uis_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned int *) pointer to data string array
//		(const unsigned int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_uis_fetch_ptr map_uia_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_uis_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uis_find_exact(m,k) map_uia_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_uia_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uia_find_exact(m,k,l) map_uia_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_uis_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uis_find_exact_lo(m,k) map_uia_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_uia_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uia_find_exact_lo(m,k,l) map_uia_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_uis_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uis_find_lo_or_near(m,k) map_uia_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_uia_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uia_find_lo_or_near(m,k,l) map_uia_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_uis_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uis_find_exact_hi(m,k) map_uia_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_uia_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uia_find_exact_hi(m,k,l) map_uia_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_uis_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uis_find_hi_or_near(m,k) map_uia_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_uia_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uia_find_hi_or_near(m,k,l) map_uia_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_uis_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uis_find(m,k,d) map_uia_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_uis_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_uis_insert(m,k) map_uia_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uis_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_uis_insert_dup(m,k) map_uia_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uis_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uis_key_copy map_uia_key_copy
//-----------------------------------------------------------------------------
// macro	map_uis_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned int *) pointer to key duplicate copy
//		(unsigned int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uis_key_dup map_uia_key_dup
//-----------------------------------------------------------------------------
// macro	map_uis_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uis_key_len map_uia_key_len
//-----------------------------------------------------------------------------
// macro	map_uis_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned int *) pointer to shared key
//		(const unsigned int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uis_key_ptr map_uia_key_ptr
//-----------------------------------------------------------------------------
// macro	map_uis_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_uis_new map_uia_new
//-----------------------------------------------------------------------------
// macro	map_uis_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_uis_store(m,d) map_uia_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uis_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_uis_store_share(m,d) map_uia_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ul_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ul_find_exact(m,k) map_ul_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_ul_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ul_find_exact_lo(m,k) map_ul_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_ul_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ul_find_lo_or_near(m,k) map_ul_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_ul_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ul_find_exact_hi(m,k) map_ul_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_ul_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ul_find_hi_or_near(m,k) map_ul_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_uls_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_uls_cmp_func map_ula_cmp_func
//-----------------------------------------------------------------------------
// macro	map_uls_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_uls_cmp_func_opt map_ula_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_uls_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_uls_fetch_copy map_ula_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_uls_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long *) pointer to duplicate data string array
//		(unsigned long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_uls_fetch_dup map_ula_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_uls_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_uls_fetch_len map_ula_fetch_len
//-----------------------------------------------------------------------------
// macro	map_uls_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long *) pointer to data string array
//		(const unsigned long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_uls_fetch_ptr map_ula_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_uls_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uls_find_exact(m,k) map_ula_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_ula_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ula_find_exact(m,k,l) map_ula_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_uls_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uls_find_exact_lo(m,k) map_ula_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_ula_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ula_find_exact_lo(m,k,l) map_ula_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_uls_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uls_find_lo_or_near(m,k) map_ula_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_ula_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ula_find_lo_or_near(m,k,l) map_ula_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_uls_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uls_find_exact_hi(m,k) map_ula_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_ula_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ula_find_exact_hi(m,k,l) map_ula_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_uls_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uls_find_hi_or_near(m,k) map_ula_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_ula_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ula_find_hi_or_near(m,k,l) map_ula_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_uls_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uls_find(m,k,d) map_ula_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_uls_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_uls_insert(m,k) map_ula_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uls_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_uls_insert_dup(m,k) map_ula_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uls_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uls_key_copy map_ula_key_copy
//-----------------------------------------------------------------------------
// macro	map_uls_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long *) pointer to key duplicate copy
//		(unsigned long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uls_key_dup map_ula_key_dup
//-----------------------------------------------------------------------------
// macro	map_uls_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uls_key_len map_ula_key_len
//-----------------------------------------------------------------------------
// macro	map_uls_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long *) pointer to shared key
//		(const unsigned long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uls_key_ptr map_ula_key_ptr
//-----------------------------------------------------------------------------
// macro	map_uls_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_uls_new map_ula_new
//-----------------------------------------------------------------------------
// macro	map_uls_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_uls_store(m,d) map_ula_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uls_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_uls_store_share(m,d) map_ula_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ull_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ull_find_exact(m,k) map_ull_find((m),(k),0)

//-----------------------------------------------------------------------------
// macro	map_ull_find_exact_lo
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ull_find_exact_lo(m,k) map_ull_find((m),(k),-1)

//-----------------------------------------------------------------------------
// macro	map_ull_find_lo_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest lower value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ull_find_lo_or_near(m,k) map_ull_find((m),(k),-2)

//-----------------------------------------------------------------------------
// macro	map_ull_find_exact_hi
//
// purpose	Find a node with an exactly matching key of the specific
//		scalar key type for this mapping.  If there is more than
//		one node of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ull_find_exact_hi(m,k) map_ull_find((m),(k),1)

//-----------------------------------------------------------------------------
// macro	map_ull_find_hi_or_near
//
// purpose	Find a node with an exactly matching key, or if no exactly
//		matching key present, a node with nearest higher value, of
//		the specific scalar key type for this mapping.  If there is
//		more than one node of the same key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
#define map_ull_find_hi_or_near(m,k) map_ull_find((m),(k),2)
//-----------------------------------------------------------------------------
// macro	map_ulls_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ulls_cmp_func map_ulla_cmp_func
//-----------------------------------------------------------------------------
// macro	map_ulls_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_ulls_cmp_func_opt map_ulla_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_ulls_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_ulls_fetch_copy map_ulla_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_ulls_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long long *) pointer to duplicate data string array
//		(unsigned long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_ulls_fetch_dup map_ulla_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_ulls_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_ulls_fetch_len map_ulla_fetch_len
//-----------------------------------------------------------------------------
// macro	map_ulls_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long long *) pointer to data string array
//		(const unsigned long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_ulls_fetch_ptr map_ulla_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_ulls_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulls_find_exact(m,k) map_ulla_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_ulla_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulla_find_exact(m,k,l) map_ulla_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_ulls_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulls_find_exact_lo(m,k) map_ulla_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_ulla_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulla_find_exact_lo(m,k,l) map_ulla_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_ulls_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulls_find_lo_or_near(m,k) map_ulla_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_ulla_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulla_find_lo_or_near(m,k,l) map_ulla_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_ulls_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulls_find_exact_hi(m,k) map_ulla_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_ulla_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulla_find_exact_hi(m,k,l) map_ulla_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_ulls_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulls_find_hi_or_near(m,k) map_ulla_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_ulla_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulla_find_hi_or_near(m,k,l) map_ulla_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_ulls_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_ulls_find(m,k,d) map_ulla_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_ulls_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_ulls_insert(m,k) map_ulla_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ulls_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_ulls_insert_dup(m,k) map_ulla_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ulls_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ulls_key_copy map_ulla_key_copy
//-----------------------------------------------------------------------------
// macro	map_ulls_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long long *) pointer to key duplicate copy
//		(unsigned long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ulls_key_dup map_ulla_key_dup
//-----------------------------------------------------------------------------
// macro	map_ulls_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ulls_key_len map_ulla_key_len
//-----------------------------------------------------------------------------
// macro	map_ulls_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long long *) pointer to shared key
//		(const unsigned long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ulls_key_ptr map_ulla_key_ptr
//-----------------------------------------------------------------------------
// macro	map_ulls_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_ulls_new map_ulla_new
//-----------------------------------------------------------------------------
// macro	map_ulls_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_ulls_store(m,d) map_ulla_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_ulls_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_ulls_store_share(m,d) map_ulla_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uss_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_uss_cmp_func map_usa_cmp_func
//-----------------------------------------------------------------------------
// macro	map_uss_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_uss_cmp_func_opt map_usa_cmp_func_opt
//-----------------------------------------------------------------------------
// macro	map_uss_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned short *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_uss_fetch_copy map_usa_fetch_copy
//-----------------------------------------------------------------------------
// macro	map_uss_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned short *) pointer to duplicate data string array
//		(unsigned short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_uss_fetch_dup map_usa_fetch_dup
//-----------------------------------------------------------------------------
// macro	map_uss_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_uss_fetch_len map_usa_fetch_len
//-----------------------------------------------------------------------------
// macro	map_uss_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned short *) pointer to data string array
//		(const unsigned short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_uss_fetch_ptr map_usa_fetch_ptr
//-----------------------------------------------------------------------------
// macro	map_uss_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uss_find_exact(m,k) map_usa_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_usa_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_usa_find_exact(m,k,l) map_usa_find((m),(k),(l),0)
//-----------------------------------------------------------------------------
// macro	map_uss_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uss_find_exact_lo(m,k) map_usa_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_usa_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_usa_find_exact_lo(m,k,l) map_usa_find((m),(k),(l),-1)
//-----------------------------------------------------------------------------
// macro	map_uss_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uss_find_lo_or_near(m,k) map_usa_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_usa_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_usa_find_lo_or_near(m,k,l) map_usa_find((m),(k),(l),-2)
//-----------------------------------------------------------------------------
// macro	map_uss_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uss_find_exact_hi(m,k) map_usa_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_usa_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_usa_find_exact_hi(m,k,l) map_usa_find((m),(k),(l),1)
//-----------------------------------------------------------------------------
// macro	map_uss_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uss_find_hi_or_near(m,k) map_usa_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_usa_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_usa_find_hi_or_near(m,k,l) map_usa_find((m),(k),(l),2)
//-----------------------------------------------------------------------------
// macro	map_uss_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_uss_find(m,k,d) map_usa_find((m),(k),~(size_t)0,(d))
//-----------------------------------------------------------------------------
// macro	map_uss_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_uss_insert(m,k) map_usa_insert((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uss_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_uss_insert_dup(m,k) map_usa_insert_dup((m),(k),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uss_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned short *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uss_key_copy map_usa_key_copy
//-----------------------------------------------------------------------------
// macro	map_uss_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned short *) pointer to key duplicate copy
//		(unsigned short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uss_key_dup map_usa_key_dup
//-----------------------------------------------------------------------------
// macro	map_uss_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uss_key_len map_usa_key_len
//-----------------------------------------------------------------------------
// macro	map_uss_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned short *) pointer to shared key
//		(const unsigned short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_uss_key_ptr map_usa_key_ptr
//-----------------------------------------------------------------------------
// macro	map_uss_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_uss_new map_usa_new
//-----------------------------------------------------------------------------
// macro	map_uss_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_uss_store(m,d) map_usa_store((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// macro	map_uss_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned short *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_uss_store_share(m,d) map_usa_store_share((m),(d),~(size_t)0)
//-----------------------------------------------------------------------------
// function	map_check (diagnostic)
//
// purpose	Check the integrity of a mapping via AVL.  This function is for
//		diagnostic purposes only.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) 0 for no error
//		(int) or-ed flags for detected errors
//-----------------------------------------------------------------------------
int
map_check (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_data_clear
// alias	map_none_store
//
// purpose	Clear the data value of the active member of a mapping so
//		that it is just like it was newly inserted with no value.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no current member to clear
//		(int)  0 = current member cleared OK
//-----------------------------------------------------------------------------
#define map_none_store(m) map_data_clear((m))

int
map_data_clear (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_data_type
//
// purpose	Return the data type of the active member of a mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no active member
//		(int)  0 = member found, no data
//		(int) >0 = member found, data type returned
//-----------------------------------------------------------------------------
int
map_data_type (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_delete
//
// purpose	Delete the currently selected member of a mapping without
//		consideration of the key type.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no current member to delete
//		(int)  1 = current member deleted OK
//-----------------------------------------------------------------------------
int
map_delete (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_destroy
//
// purpose	Destroy a mapping.  Delete all the nodes and free the mapping.
//		The mapping handle becomes invalid and must not be used again.
//
// arguments	1 (MAP) mapping handle
//
// returns	(long) -2 = error, no map
//		(long) number of members deleted
//
// note		In UNIX and similar virtual memory systems, if the process
//		is near the end of its execution, it is likely not necessary
//		to free the remaining space, as virtual memory cleanup will
//		remove it more efficiently.
//-----------------------------------------------------------------------------
long
map_destroy (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_empty
//
// purpose	Delete all the members of a mapping without consideration of
//		the key type.
//
// arguments	1 (MAP) mapping handle
//
// returns	(long) -2 = error, no map
//		(long) number of members deleted
//-----------------------------------------------------------------------------
long
map_empty (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_first
//
// purpose	Select the first member of a mapping as the active one.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no members in map
//		(int)  0 = member found, no data
//		(int) >0 = member found, data type returned
//-----------------------------------------------------------------------------
int
map_first (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_forward
//
// purpose	Move the current member selection of a mapping one step in
//		the forward direction.	If there is no current member, then
//		select the first member.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no member found
//		(int)  0 = member found, no data
//		(int) >0 = member found, data type returned
//-----------------------------------------------------------------------------
int
map_forward (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_free_node (internal)
//
// purpose	Free a node, including data content where applicable.  Since
//		it is not necessary to know the node or data size in order to
//		free it, this code only needs to get the pointers for freeing.
//
// arguments	1 (void *) pointer to variant node type
//
// returns	(void)
//-----------------------------------------------------------------------------
void
map_free_node (
    void *	arg_node
    )
;
//-----------------------------------------------------------------------------
// function	map_key_type
//
// purpose	Return the key type of the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) >0 = key type returned
//-----------------------------------------------------------------------------
int
map_key_type (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_last
//
// purpose	Select the last member of a mapping as the active one.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no members in map
//		(int)  0 = member found, no data
//		(int) >0 = member found, data type returned
//-----------------------------------------------------------------------------
int
map_last (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_reverse
//
// purpose	Move the current member selection of a mapping one step in
//		the reverse direction.	If there is no current member, then
//		select the last member.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no member found
//		(int)  0 = member found, no data
//		(int) >0 = member found, data type returned
//-----------------------------------------------------------------------------
int
map_reverse (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_same_new
//
// purpose	Make a new mapping with the same key type as another given
//		mapping.
//
// arguments	1 (MAP) mapping handle of guide mapping
//
// returns	(MAP) NULL: error
//		(MAP) handle of new mapping
//
// warning	This will cause the compare function from the given mapping to
//		also be used in the new mapping.  If the compare function is
//		stateful or was dynamically loaded and gets unloaded, results
//		are unpredictable, and likely disasterous.
//-----------------------------------------------------------------------------
MAP
map_same_new (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_unselect
//
// purpose	Clear the current member selection and make the mapping have
//		no currently selected member.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no member was selected before
//		(int)  1 = an member was selected before
//-----------------------------------------------------------------------------
int
map_unselect (
    MAP		arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_digits (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_feature (
    MAP		arg_map
    ,
    int		arg_state
    ,
    int		arg_flag
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const char *
				 ,
				 size_t
				 ,
				 const char *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const char *
				,
				size_t
				,
				const char *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_lower (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_paths (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_spaces (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_upper (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_ca *)
//		2 (struct map_link_ca *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ca_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ca_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_ca *)
//		2 (struct map_link_ca *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const char *) key 1
//		2 (size_t) length of key 1
//		3 (const char *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ca_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ca_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_ca *)
//		2 (struct map_link_ca *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const char *) key 1
//		2 (size_t) length of key 1
//		3 (const char *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ca_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ca_compare_str (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This compare function provides for optional features to do the
//		comparison in varied ways.  These include:
//
//		lower:	Compare alphabetic characters as if converted to lower
//			case (even if compared to non-alphabetic).
//
//		upper:	Compare alphabetic characters as if converted to upper
//			case (even if compared to non-alphabetic).
//
//		digits:	Compare sequences of 1 or more digits by prefixing the
//			shorter sequence with 0 digits to make the comparison
//			work on the numeric value.
//
//		spaces:	Compare sequences of 1 or more non-graphical characters
//			as being equal to a single binary 0 character.
//
//		paths:	Compare file path directory separator characters as
//			being lower in collating order than other characters.
//
//		Options are set in the map_ca_cmp_func_opt function.
//
// arguments	1 (struct map_link_ca *)
//		2 (struct map_link_ca *)
//		3 (AVL *) AVL link pointer
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ca_compare_str (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ca_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ca_fetch_copy (
    MAP			arg_map
    ,
    char *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(char *) pointer to duplicate data string array
//		(char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
char *
map_ca_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ca_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const char *) pointer to data string array
//		(const char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const char *
map_ca_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_ca_find (
    MAP			arg_map
    ,
    const char *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_ca_insert (
    MAP			arg_map
    ,
    const char *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_ca_insert_dup (
    MAP			arg_map
    ,
    const char *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ca_key_copy (
    MAP			arg_map
    ,
    char *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(char *) pointer to key duplicate copy
//		(char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
char *
map_ca_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ca_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ca_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ca_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const char *) pointer to shared key
//		(const char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const char *
map_ca_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ca_new ()
;
//-----------------------------------------------------------------------------
// function	map_ca_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const char *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ca_store (
    MAP			arg_map
    ,
    const char *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ca_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (char *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ca_store_share (
    MAP			arg_map
    ,
    char *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function     map_d_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(double,double) function ptr
//
// comparison	1 (double) key value 1
//  arguments	2 (double) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_d_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( double, double )
    )
;
//-----------------------------------------------------------------------------
// function     map_d_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(double,double,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (double) key value 1
//  arguments	2 (double) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_d_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( double, double, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_d_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(double) data value returned
//		(double) 0 cast to the data type if error
//-----------------------------------------------------------------------------
double
map_d_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_d_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(double) data value returned
//		(double) 0 cast to the data type if error
//-----------------------------------------------------------------------------
double
map_d_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_d_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_d_find (
    MAP			arg_map
    ,
    double		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_d_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_d_insert (
    MAP			arg_map
    ,
    double		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_d_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_d_insert_dup (
    MAP			arg_map
    ,
    double		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_d_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(double) key value
//		(double) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
double
map_d_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_d_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_d_new ()
;
//-----------------------------------------------------------------------------
// function	map_d_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (double) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_d_store (
    MAP			arg_map
    ,
    double		arg_data
    )
;
//-----------------------------------------------------------------------------
// function     map_ld_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(long double,long double) function ptr
//
// comparison	1 (long double) key value 1
//  arguments	2 (long double) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ld_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( long double, long double )
    )
;
//-----------------------------------------------------------------------------
// function     map_ld_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(long double,long double,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (long double) key value 1
//  arguments	2 (long double) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ld_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( long double, long double, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_ld_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(long double) data value returned
//		(long double) 0 cast to the data type if error
//-----------------------------------------------------------------------------
long double
map_ld_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ld_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(long double) data value returned
//		(long double) 0 cast to the data type if error
//-----------------------------------------------------------------------------
long double
map_ld_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ld_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_ld_find (
    MAP			arg_map
    ,
    long double		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_ld_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_ld_insert (
    MAP			arg_map
    ,
    long double		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ld_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_ld_insert_dup (
    MAP			arg_map
    ,
    long double		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ld_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(long double) key value
//		(long double) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
long double
map_ld_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ld_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ld_new ()
;
//-----------------------------------------------------------------------------
// function	map_ld_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (long double) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ld_store (
    MAP			arg_map
    ,
    long double		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_map_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(MAP) data value returned
//		(MAP) 0 cast to the data type if error
//-----------------------------------------------------------------------------
MAP
map_map_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_map_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (MAP) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_map_store (
    MAP			arg_map
    ,
    MAP		arg_data
    )
;
//-----------------------------------------------------------------------------
// function     map_ptr_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(void *,void *) function ptr
//
// comparison	1 (void *) key value 1
//  arguments	2 (void *) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ptr_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( void *, void * )
    )
;
//-----------------------------------------------------------------------------
// function     map_ptr_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(void *,void *,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (void *) key value 1
//  arguments	2 (void *) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ptr_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( void *, void *, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_ptr_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(void *) data value returned
//		(void *) 0 cast to the data type if error
//-----------------------------------------------------------------------------
void *
map_ptr_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ptr_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_ptr_find (
    MAP			arg_map
    ,
    void *		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_ptr_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_ptr_insert (
    MAP			arg_map
    ,
    void *		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ptr_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_ptr_insert_dup (
    MAP			arg_map
    ,
    void *		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ptr_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(void *) key value
//		(void *) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
void *
map_ptr_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ptr_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ptr_new ()
;
//-----------------------------------------------------------------------------
// function	map_ptr_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (void *) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ptr_store (
    MAP			arg_map
    ,
    void *		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_digits (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_feature (
    MAP		arg_map
    ,
    int		arg_state
    ,
    int		arg_flag
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const signed char *
				 ,
				 size_t
				 ,
				 const signed char *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const signed char *
				,
				size_t
				,
				const signed char *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_lower (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_paths (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_spaces (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sca_cmp_upper (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_sca *)
//		2 (struct map_link_sca *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sca_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sca_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_sca *)
//		2 (struct map_link_sca *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed char *) key 1
//		2 (size_t) length of key 1
//		3 (const signed char *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sca_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sca_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_sca *)
//		2 (struct map_link_sca *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed char *) key 1
//		2 (size_t) length of key 1
//		3 (const signed char *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sca_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sca_compare_str (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This compare function provides for optional features to do the
//		comparison in varied ways.  These include:
//
//		lower:	Compare alphabetic characters as if converted to lower
//			case (even if compared to non-alphabetic).
//
//		upper:	Compare alphabetic characters as if converted to upper
//			case (even if compared to non-alphabetic).
//
//		digits:	Compare sequences of 1 or more digits by prefixing the
//			shorter sequence with 0 digits to make the comparison
//			work on the numeric value.
//
//		spaces:	Compare sequences of 1 or more non-graphical characters
//			as being equal to a single binary 0 character.
//
//		paths:	Compare file path directory separator characters as
//			being lower in collating order than other characters.
//
//		Options are set in the map_sca_cmp_func_opt function.
//
// arguments	1 (struct map_link_sca *)
//		2 (struct map_link_sca *)
//		3 (AVL *) AVL link pointer
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sca_compare_str (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sca_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed char *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_sca_fetch_copy (
    MAP			arg_map
    ,
    signed char *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed char *) pointer to duplicate data string array
//		(signed char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
signed char *
map_sca_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_sca_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed char *) pointer to data string array
//		(const signed char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const signed char *
map_sca_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_sca_find (
    MAP			arg_map
    ,
    const signed char *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_sca_insert (
    MAP			arg_map
    ,
    const signed char *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_sca_insert_dup (
    MAP			arg_map
    ,
    const signed char *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed char *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_sca_key_copy (
    MAP			arg_map
    ,
    signed char *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed char *) pointer to key duplicate copy
//		(signed char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
signed char *
map_sca_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sca_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_sca_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sca_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed char *) pointer to shared key
//		(const signed char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const signed char *
map_sca_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_sca_new ()
;
//-----------------------------------------------------------------------------
// function	map_sca_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed char *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sca_store (
    MAP			arg_map
    ,
    const signed char *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sca_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed char *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sca_store_share (
    MAP			arg_map
    ,
    signed char *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function     map_si_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(signed int,signed int) function ptr
//
// comparison	1 (signed int) key value 1
//  arguments	2 (signed int) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_si_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( signed int, signed int )
    )
;
//-----------------------------------------------------------------------------
// function     map_si_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(signed int,signed int,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (signed int) key value 1
//  arguments	2 (signed int) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_si_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( signed int, signed int, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_si_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed int) data value returned
//		(signed int) 0 cast to the data type if error
//-----------------------------------------------------------------------------
signed int
map_si_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_si_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed int) data value returned
//		(signed int) 0 cast to the data type if error
//-----------------------------------------------------------------------------
signed int
map_si_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_si_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_si_find (
    MAP			arg_map
    ,
    signed int		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_si_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_si_insert (
    MAP			arg_map
    ,
    signed int		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_si_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_si_insert_dup (
    MAP			arg_map
    ,
    signed int		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_si_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed int) key value
//		(signed int) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
signed int
map_si_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_si_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_si_new ()
;
//-----------------------------------------------------------------------------
// function	map_si_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_si_store (
    MAP			arg_map
    ,
    signed int		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_sia_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sia_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const signed int *
				 ,
				 size_t
				 ,
				 const signed int *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sia_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const signed int *
				,
				size_t
				,
				const signed int *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_sia *)
//		2 (struct map_link_sia *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sia_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sia_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_sia *)
//		2 (struct map_link_sia *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed int *) key 1
//		2 (size_t) length of key 1
//		3 (const signed int *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sia_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sia_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_sia *)
//		2 (struct map_link_sia *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed int *) key 1
//		2 (size_t) length of key 1
//		3 (const signed int *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sia_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sia_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_sia_fetch_copy (
    MAP			arg_map
    ,
    signed int *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed int *) pointer to duplicate data string array
//		(signed int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
signed int *
map_sia_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_sia_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed int *) pointer to data string array
//		(const signed int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const signed int *
map_sia_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_sia_find (
    MAP			arg_map
    ,
    const signed int *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_sia_insert (
    MAP			arg_map
    ,
    const signed int *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_sia_insert_dup (
    MAP			arg_map
    ,
    const signed int *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_sia_key_copy (
    MAP			arg_map
    ,
    signed int *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed int *) pointer to key duplicate copy
//		(signed int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
signed int *
map_sia_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sia_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_sia_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sia_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed int *) pointer to shared key
//		(const signed int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const signed int *
map_sia_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_sia_new ()
;
//-----------------------------------------------------------------------------
// function	map_sia_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sia_store (
    MAP			arg_map
    ,
    const signed int *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sia_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed int *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sia_store_share (
    MAP			arg_map
    ,
    signed int *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function     map_sl_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(signed long,signed long) function ptr
//
// comparison	1 (signed long) key value 1
//  arguments	2 (signed long) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_sl_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( signed long, signed long )
    )
;
//-----------------------------------------------------------------------------
// function     map_sl_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(signed long,signed long,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (signed long) key value 1
//  arguments	2 (signed long) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_sl_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( signed long, signed long, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_sl_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed long) data value returned
//		(signed long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
signed long
map_sl_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sl_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed long) data value returned
//		(signed long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
signed long
map_sl_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sl_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_sl_find (
    MAP			arg_map
    ,
    signed long		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_sl_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_sl_insert (
    MAP			arg_map
    ,
    signed long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_sl_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_sl_insert_dup (
    MAP			arg_map
    ,
    signed long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_sl_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed long) key value
//		(signed long) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
signed long
map_sl_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sl_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_sl_new ()
;
//-----------------------------------------------------------------------------
// function	map_sl_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sl_store (
    MAP			arg_map
    ,
    signed long		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_sla_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sla_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const signed long *
				 ,
				 size_t
				 ,
				 const signed long *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_sla_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const signed long *
				,
				size_t
				,
				const signed long *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_sla *)
//		2 (struct map_link_sla *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sla_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sla_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_sla *)
//		2 (struct map_link_sla *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed long *) key 1
//		2 (size_t) length of key 1
//		3 (const signed long *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sla_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sla_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_sla *)
//		2 (struct map_link_sla *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed long *) key 1
//		2 (size_t) length of key 1
//		3 (const signed long *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_sla_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_sla_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_sla_fetch_copy (
    MAP			arg_map
    ,
    signed long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long *) pointer to duplicate data string array
//		(signed long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
signed long *
map_sla_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_sla_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long *) pointer to data string array
//		(const signed long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const signed long *
map_sla_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_sla_find (
    MAP			arg_map
    ,
    const signed long *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_sla_insert (
    MAP			arg_map
    ,
    const signed long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_sla_insert_dup (
    MAP			arg_map
    ,
    const signed long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_sla_key_copy (
    MAP			arg_map
    ,
    signed long *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long *) pointer to key duplicate copy
//		(signed long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
signed long *
map_sla_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sla_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_sla_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sla_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long *) pointer to shared key
//		(const signed long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const signed long *
map_sla_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_sla_new ()
;
//-----------------------------------------------------------------------------
// function	map_sla_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sla_store (
    MAP			arg_map
    ,
    const signed long *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_sla_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sla_store_share (
    MAP			arg_map
    ,
    signed long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function     map_sll_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(signed long long,signed long long) function ptr
//
// comparison	1 (signed long long) key value 1
//  arguments	2 (signed long long) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_sll_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( signed long long, signed long long )
    )
;
//-----------------------------------------------------------------------------
// function     map_sll_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(signed long long,signed long long,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (signed long long) key value 1
//  arguments	2 (signed long long) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_sll_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( signed long long, signed long long, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_sll_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed long long) data value returned
//		(signed long long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
signed long long
map_sll_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sll_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed long long) data value returned
//		(signed long long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
signed long long
map_sll_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sll_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_sll_find (
    MAP			arg_map
    ,
    signed long long		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_sll_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_sll_insert (
    MAP			arg_map
    ,
    signed long long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_sll_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_sll_insert_dup (
    MAP			arg_map
    ,
    signed long long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_sll_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(signed long long) key value
//		(signed long long) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
signed long long
map_sll_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_sll_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_sll_new ()
;
//-----------------------------------------------------------------------------
// function	map_sll_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sll_store (
    MAP			arg_map
    ,
    signed long long		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_slla_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_slla_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const signed long long *
				 ,
				 size_t
				 ,
				 const signed long long *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_slla_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const signed long long *
				,
				size_t
				,
				const signed long long *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_slla *)
//		2 (struct map_link_slla *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_slla_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_slla_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_slla *)
//		2 (struct map_link_slla *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed long long *) key 1
//		2 (size_t) length of key 1
//		3 (const signed long long *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_slla_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_slla_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_slla *)
//		2 (struct map_link_slla *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed long long *) key 1
//		2 (size_t) length of key 1
//		3 (const signed long long *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_slla_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_slla_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_slla_fetch_copy (
    MAP			arg_map
    ,
    signed long long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long long *) pointer to duplicate data string array
//		(signed long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
signed long long *
map_slla_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_slla_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long long *) pointer to data string array
//		(const signed long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const signed long long *
map_slla_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_slla_find (
    MAP			arg_map
    ,
    const signed long long *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_slla_insert (
    MAP			arg_map
    ,
    const signed long long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_slla_insert_dup (
    MAP			arg_map
    ,
    const signed long long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_slla_key_copy (
    MAP			arg_map
    ,
    signed long long *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed long long *) pointer to key duplicate copy
//		(signed long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
signed long long *
map_slla_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_slla_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_slla_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_slla_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed long long *) pointer to shared key
//		(const signed long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const signed long long *
map_slla_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_slla_new ()
;
//-----------------------------------------------------------------------------
// function	map_slla_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed long long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_slla_store (
    MAP			arg_map
    ,
    const signed long long *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_slla_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_slla_store_share (
    MAP			arg_map
    ,
    signed long long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const signed short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ssa_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const signed short *
				 ,
				 size_t
				 ,
				 const signed short *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const signed short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const signed short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ssa_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const signed short *
				,
				size_t
				,
				const signed short *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_ssa *)
//		2 (struct map_link_ssa *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ssa_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ssa_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_ssa *)
//		2 (struct map_link_ssa *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed short *) key 1
//		2 (size_t) length of key 1
//		3 (const signed short *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ssa_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ssa_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_ssa *)
//		2 (struct map_link_ssa *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const signed short *) key 1
//		2 (size_t) length of key 1
//		3 (const signed short *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ssa_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ssa_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed short *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ssa_fetch_copy (
    MAP			arg_map
    ,
    signed short *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed short *) pointer to duplicate data string array
//		(signed short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
signed short *
map_ssa_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ssa_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed short *) pointer to data string array
//		(const signed short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const signed short *
map_ssa_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_ssa_find (
    MAP			arg_map
    ,
    const signed short *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_ssa_insert (
    MAP			arg_map
    ,
    const signed short *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_ssa_insert_dup (
    MAP			arg_map
    ,
    const signed short *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (signed short *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ssa_key_copy (
    MAP			arg_map
    ,
    signed short *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed short *) pointer to key duplicate copy
//		(signed short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
signed short *
map_ssa_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ssa_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ssa_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ssa_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const signed short *) pointer to shared key
//		(const signed short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const signed short *
map_ssa_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ssa_new ()
;
//-----------------------------------------------------------------------------
// function	map_ssa_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed short *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ssa_store (
    MAP			arg_map
    ,
    const signed short *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ssa_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (signed short *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ssa_store_share (
    MAP			arg_map
    ,
    signed short *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_digits (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_feature (
    MAP		arg_map
    ,
    int		arg_state
    ,
    int		arg_flag
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const unsigned char *
				 ,
				 size_t
				 ,
				 const unsigned char *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const unsigned char *
				,
				size_t
				,
				const unsigned char *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_lower (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_paths (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_spaces (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uca_cmp_upper (
    MAP		arg_map
    ,
    int		arg_change
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_uca *)
//		2 (struct map_link_uca *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_uca_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_uca_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_uca *)
//		2 (struct map_link_uca *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned char *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned char *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_uca_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_uca_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_uca *)
//		2 (struct map_link_uca *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned char *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned char *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_uca_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_uca_compare_str (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This compare function provides for optional features to do the
//		comparison in varied ways.  These include:
//
//		lower:	Compare alphabetic characters as if converted to lower
//			case (even if compared to non-alphabetic).
//
//		upper:	Compare alphabetic characters as if converted to upper
//			case (even if compared to non-alphabetic).
//
//		digits:	Compare sequences of 1 or more digits by prefixing the
//			shorter sequence with 0 digits to make the comparison
//			work on the numeric value.
//
//		spaces:	Compare sequences of 1 or more non-graphical characters
//			as being equal to a single binary 0 character.
//
//		paths:	Compare file path directory separator characters as
//			being lower in collating order than other characters.
//
//		Options are set in the map_uca_cmp_func_opt function.
//
// arguments	1 (struct map_link_uca *)
//		2 (struct map_link_uca *)
//		3 (AVL *) AVL link pointer
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_uca_compare_str (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_uca_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned char *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_uca_fetch_copy (
    MAP			arg_map
    ,
    unsigned char *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned char *) pointer to duplicate data string array
//		(unsigned char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
unsigned char *
map_uca_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_uca_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned char *) pointer to data string array
//		(const unsigned char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const unsigned char *
map_uca_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_uca_find (
    MAP			arg_map
    ,
    const unsigned char *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_uca_insert (
    MAP			arg_map
    ,
    const unsigned char *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_uca_insert_dup (
    MAP			arg_map
    ,
    const unsigned char *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned char *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_uca_key_copy (
    MAP			arg_map
    ,
    unsigned char *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned char *) pointer to key duplicate copy
//		(unsigned char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
unsigned char *
map_uca_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_uca_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_uca_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_uca_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned char *) pointer to shared key
//		(const unsigned char *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const unsigned char *
map_uca_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_uca_new ()
;
//-----------------------------------------------------------------------------
// function	map_uca_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned char *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_uca_store (
    MAP			arg_map
    ,
    const unsigned char *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uca_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned char *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_uca_store_share (
    MAP			arg_map
    ,
    unsigned char *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function     map_ui_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(unsigned int,unsigned int) function ptr
//
// comparison	1 (unsigned int) key value 1
//  arguments	2 (unsigned int) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ui_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( unsigned int, unsigned int )
    )
;
//-----------------------------------------------------------------------------
// function     map_ui_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(unsigned int,unsigned int,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (unsigned int) key value 1
//  arguments	2 (unsigned int) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ui_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( unsigned int, unsigned int, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_ui_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned int) data value returned
//		(unsigned int) 0 cast to the data type if error
//-----------------------------------------------------------------------------
unsigned int
map_ui_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ui_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned int) data value returned
//		(unsigned int) 0 cast to the data type if error
//-----------------------------------------------------------------------------
unsigned int
map_ui_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ui_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_ui_find (
    MAP			arg_map
    ,
    unsigned int		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_ui_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_ui_insert (
    MAP			arg_map
    ,
    unsigned int		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ui_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_ui_insert_dup (
    MAP			arg_map
    ,
    unsigned int		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ui_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned int) key value
//		(unsigned int) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
unsigned int
map_ui_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ui_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ui_new ()
;
//-----------------------------------------------------------------------------
// function	map_ui_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ui_store (
    MAP			arg_map
    ,
    unsigned int		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_uia_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uia_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const unsigned int *
				 ,
				 size_t
				 ,
				 const unsigned int *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned int *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned int *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_uia_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const unsigned int *
				,
				size_t
				,
				const unsigned int *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_uia *)
//		2 (struct map_link_uia *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_uia_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_uia_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_uia *)
//		2 (struct map_link_uia *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned int *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned int *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_uia_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_uia_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_uia *)
//		2 (struct map_link_uia *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned int *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned int *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_uia_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_uia_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_uia_fetch_copy (
    MAP			arg_map
    ,
    unsigned int *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned int *) pointer to duplicate data string array
//		(unsigned int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
unsigned int *
map_uia_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_uia_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned int *) pointer to data string array
//		(const unsigned int *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const unsigned int *
map_uia_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_uia_find (
    MAP			arg_map
    ,
    const unsigned int *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_uia_insert (
    MAP			arg_map
    ,
    const unsigned int *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_uia_insert_dup (
    MAP			arg_map
    ,
    const unsigned int *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_uia_key_copy (
    MAP			arg_map
    ,
    unsigned int *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned int *) pointer to key duplicate copy
//		(unsigned int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
unsigned int *
map_uia_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_uia_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_uia_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_uia_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned int *) pointer to shared key
//		(const unsigned int *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const unsigned int *
map_uia_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_uia_new ()
;
//-----------------------------------------------------------------------------
// function	map_uia_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned int *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_uia_store (
    MAP			arg_map
    ,
    const unsigned int *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_uia_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned int *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_uia_store_share (
    MAP			arg_map
    ,
    unsigned int *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function     map_ul_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(unsigned long,unsigned long) function ptr
//
// comparison	1 (unsigned long) key value 1
//  arguments	2 (unsigned long) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ul_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( unsigned long, unsigned long )
    )
;
//-----------------------------------------------------------------------------
// function     map_ul_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(unsigned long,unsigned long,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (unsigned long) key value 1
//  arguments	2 (unsigned long) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ul_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( unsigned long, unsigned long, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_ul_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned long) data value returned
//		(unsigned long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
unsigned long
map_ul_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ul_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned long) data value returned
//		(unsigned long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
unsigned long
map_ul_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ul_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_ul_find (
    MAP			arg_map
    ,
    unsigned long		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_ul_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_ul_insert (
    MAP			arg_map
    ,
    unsigned long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ul_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_ul_insert_dup (
    MAP			arg_map
    ,
    unsigned long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ul_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned long) key value
//		(unsigned long) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
unsigned long
map_ul_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ul_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ul_new ()
;
//-----------------------------------------------------------------------------
// function	map_ul_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ul_store (
    MAP			arg_map
    ,
    unsigned long		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_ula_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ula_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const unsigned long *
				 ,
				 size_t
				 ,
				 const unsigned long *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ula_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const unsigned long *
				,
				size_t
				,
				const unsigned long *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_ula *)
//		2 (struct map_link_ula *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ula_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ula_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_ula *)
//		2 (struct map_link_ula *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned long *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned long *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ula_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ula_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_ula *)
//		2 (struct map_link_ula *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned long *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned long *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ula_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ula_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ula_fetch_copy (
    MAP			arg_map
    ,
    unsigned long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long *) pointer to duplicate data string array
//		(unsigned long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
unsigned long *
map_ula_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ula_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long *) pointer to data string array
//		(const unsigned long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const unsigned long *
map_ula_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_ula_find (
    MAP			arg_map
    ,
    const unsigned long *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_ula_insert (
    MAP			arg_map
    ,
    const unsigned long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_ula_insert_dup (
    MAP			arg_map
    ,
    const unsigned long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ula_key_copy (
    MAP			arg_map
    ,
    unsigned long *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long *) pointer to key duplicate copy
//		(unsigned long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
unsigned long *
map_ula_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ula_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ula_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ula_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long *) pointer to shared key
//		(const unsigned long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const unsigned long *
map_ula_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ula_new ()
;
//-----------------------------------------------------------------------------
// function	map_ula_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ula_store (
    MAP			arg_map
    ,
    const unsigned long *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ula_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ula_store_share (
    MAP			arg_map
    ,
    unsigned long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function     map_ull_cmp_func
//
// purpose      Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(unsigned long long,unsigned long long) function ptr
//
// comparison	1 (unsigned long long) key value 1
//  arguments	2 (unsigned long long) key value 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ull_cmp_func (
    MAP         arg_map
    ,
    int		(* arg_compare)( unsigned long long, unsigned long long )
    )
;
//-----------------------------------------------------------------------------
// function     map_ull_cmp_func_opt
//
// purpose      Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping to set function for
//		2 ((*)(unsigned long long,unsigned long long,int) function ptr
//		3 (int) option to pass along
//
// comparison	1 (unsigned long long) key value 1
//  arguments	2 (unsigned long long) key value 2
//		3 (int) option being passed along
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1  > key 2
//
// returns      (int) -1 : error
//              (int)  0 : OK
//-----------------------------------------------------------------------------
int
map_ull_cmp_func_opt (
    MAP         arg_map
    ,
    int		(* arg_compare)( unsigned long long, unsigned long long, int )
    ,
    int		arg_opt
    )
;
//-----------------------------------------------------------------------------
// function	map_ull_fetch
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value.  There is no means to indicate an error.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned long long) data value returned
//		(unsigned long long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
unsigned long long
map_ull_fetch (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ull_fetch_cast
//
// purpose	Fetch the data from the current node in the mapping and return
//		directly as a value cast or converted to the type expected by
//		caller if the stored data value is a compatible type.  There
//		is no means to indicate an error.  Valid values may be changed
//		depending on the difference in sign and/or scale of the value
//		and the target type.  An error will result in the value 0.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned long long) data value returned
//		(unsigned long long) 0 cast to the data type if error
//-----------------------------------------------------------------------------
unsigned long long
map_ull_fetch_cast (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ull_find
//
// purpose	Find a node with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest node in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no node found
//		(int)  0 = found a node with no data
//		(int) data type of found node
//-----------------------------------------------------------------------------
int
map_ull_find (
    MAP			arg_map
    ,
    unsigned long long		arg_key
    ,
    int			arg_dir_near
    )
;
//-----------------------------------------------------------------------------
// function	map_ull_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found node
//-----------------------------------------------------------------------------
int
map_ull_insert (
    MAP			arg_map
    ,
    unsigned long long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ull_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_ull_insert_dup (
    MAP			arg_map
    ,
    unsigned long long		arg_key
    )
;
//-----------------------------------------------------------------------------
// function	map_ull_key
//
// purpose	Get the key value of the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// return	(unsigned long long) key value
//		(unsigned long long) 0 cast to the key type of the tree
//-----------------------------------------------------------------------------
unsigned long long
map_ull_key (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ull_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ull_new ()
;
//-----------------------------------------------------------------------------
// function	map_ull_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ull_store (
    MAP			arg_map
    ,
    unsigned long long		arg_data
    )
;
//-----------------------------------------------------------------------------
// function	map_ulla_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ulla_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const unsigned long long *
				 ,
				 size_t
				 ,
				 const unsigned long long *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned long long *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned long long *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ulla_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const unsigned long long *
				,
				size_t
				,
				const unsigned long long *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_ulla *)
//		2 (struct map_link_ulla *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ulla_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ulla_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_ulla *)
//		2 (struct map_link_ulla *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned long long *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned long long *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ulla_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ulla_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_ulla *)
//		2 (struct map_link_ulla *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned long long *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned long long *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_ulla_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_ulla_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ulla_fetch_copy (
    MAP			arg_map
    ,
    unsigned long long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long long *) pointer to duplicate data string array
//		(unsigned long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
unsigned long long *
map_ulla_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_ulla_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long long *) pointer to data string array
//		(const unsigned long long *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const unsigned long long *
map_ulla_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_ulla_find (
    MAP			arg_map
    ,
    const unsigned long long *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_ulla_insert (
    MAP			arg_map
    ,
    const unsigned long long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_ulla_insert_dup (
    MAP			arg_map
    ,
    const unsigned long long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ulla_key_copy (
    MAP			arg_map
    ,
    unsigned long long *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned long long *) pointer to key duplicate copy
//		(unsigned long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
unsigned long long *
map_ulla_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ulla_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_ulla_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_ulla_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long long *) pointer to shared key
//		(const unsigned long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const unsigned long long *
map_ulla_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_ulla_new ()
;
//-----------------------------------------------------------------------------
// function	map_ulla_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ulla_store (
    MAP			arg_map
    ,
    const unsigned long long *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_ulla_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned long long *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_ulla_store_share (
    MAP			arg_map
    ,
    unsigned long long *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const unsigned short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_usa_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const unsigned short *
				 ,
				 size_t
				 ,
				 const unsigned short *
				 ,
				 size_t
				 )
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const unsigned short *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const unsigned short *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_usa_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const unsigned short *
				,
				size_t
				,
				const unsigned short *
				,
				size_t
				)
    ,
    int		arg_opt
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_usa *)
//		2 (struct map_link_usa *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_usa_compare (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_usa_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_usa *)
//		2 (struct map_link_usa *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned short *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned short *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_usa_compare_alt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_usa_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_usa *)
//		2 (struct map_link_usa *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const unsigned short *) key 1
//		2 (size_t) length of key 1
//		3 (const unsigned short *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_usa_compare_alt_opt (
    avl_link *		arg_one
    ,
    avl_link *		arg_two
    ,
    AVL *		arg_avl
    )
;
//-----------------------------------------------------------------------------
// function	map_usa_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned short *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_usa_fetch_copy (
    MAP			arg_map
    ,
    unsigned short *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned short *) pointer to duplicate data string array
//		(unsigned short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
unsigned short *
map_usa_fetch_dup (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_usa_fetch_len (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned short *) pointer to data string array
//		(const unsigned short *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const unsigned short *
map_usa_fetch_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_usa_find (
    MAP			arg_map
    ,
    const unsigned short *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_usa_insert (
    MAP			arg_map
    ,
    const unsigned short *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_usa_insert_dup (
    MAP			arg_map
    ,
    const unsigned short *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned short *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_usa_key_copy (
    MAP			arg_map
    ,
    unsigned short *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned short *) pointer to key duplicate copy
//		(unsigned short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
unsigned short *
map_usa_key_dup (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_usa_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_usa_key_len (
    MAP			arg_map
    )
;
//-----------------------------------------------------------------------------
// function	map_usa_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned short *) pointer to shared key
//		(const unsigned short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const unsigned short *
map_usa_key_ptr (
    MAP			arg_map
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_usa_new ()
;
//-----------------------------------------------------------------------------
// function	map_usa_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned short *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_usa_store (
    MAP			arg_map
    ,
    const unsigned short *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// function	map_usa_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (unsigned short *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_usa_store_share (
    MAP			arg_map
    ,
    unsigned short *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
;
//-----------------------------------------------------------------------------
// macro	map_from_avl
//
// purpose	Given a pointer to the avl_root structure, yield a pointer to
//		the map_root structure that contains it.  This requires a
//		backwards offset calculation and may result in alignment
//		warnings on some platforms.
//-----------------------------------------------------------------------------
#define map_from_avl(p) ((map_p)((void*)(((char*)(p))-((size_t)(&(((map_p)0)->avl_tree))))))

//-----------------------------------------------------------------------------
// macro	map_cmp_func
//
// purpose	Given a pointer to the avl_tree member (e.g. what the avl layer
//		passes as the 3rd argument in the avl comparison function),
//		yield the function pointer to the alternate comparison function
//		set up for that avl_tree.
//-----------------------------------------------------------------------------
#define map_cmp_func(p) (map_from_avl((p))->cmp_func)

//-----------------------------------------------------------------------------
// end of map.h
//-----------------------------------------------------------------------------

#endif /* __MAP_H__ */

